@@ -256,12 +256,13 @@ class RoomManager:
                                 file_type,
                                 file_format,
                                 document,
+                                room,
                             )
                         )
                         # update the document when file changes
                         if file_id not in self.watchers:
                             self.watchers[file_id] = create_task(
-                                self.watch_file(file_format, file_id, document),
+                                self.watch_file(file_format, file_id, document, room),
                                 self.task_group,
                             )
 
@@ -317,7 +318,13 @@ class RoomManager:
             document.path = file_path
         return file_path
 
-    async def watch_file(self, file_format: str, file_id: str, document: YBaseDoc) -> None:
+    async def watch_file(
+        self,
+        file_format: str,
+        file_id: str,
+        document: YBaseDoc,
+        room: YRoom,
+    ) -> None:
         file_path = await self.get_file_path(file_id, document)
         assert file_path is not None
         logger.info("Watching file", path=file_path)
@@ -332,11 +339,17 @@ class RoomManager:
                 self.file_id.unwatch(file_path, watcher)
                 file_path = new_file_path
                 # break
-            await self.maybe_load_file(file_format, file_path, file_id)
+            await self.maybe_load_file(file_format, file_path, file_id, room)
         if file_id in self.watchers:
             del self.watchers[file_id]
 
-    async def maybe_load_file(self, file_format: str, file_path: str, file_id: str) -> None:
+    async def maybe_load_file(
+        self,
+        file_format: str,
+        file_path: str,
+        file_id: str,
+        room: YRoom,
+    ) -> None:
         model = await self.contents.read_content(file_path, False)
         # do nothing if the file was saved by us
         assert model.last_modified is not None
@@ -347,10 +360,19 @@ class RoomManager:
             documents = [v for k, v in self.documents.items() if k.split(":", 2)[2] == file_id]
             for document in documents:
                 document.source = model.content
+                assert room.ystore is not None
+                await room.ystore.encode_state_as_update(room.ydoc)
             self.last_modified[file_id] = to_datetime(model.last_modified)
 
     def on_document_change(
-        self, file_id: str, file_type: str, file_format: str, document: YBaseDoc, target, event
+        self,
+        file_id: str,
+        file_type: str,
+        file_format: str,
+        document: YBaseDoc,
+        room: YRoom,
+        target,
+        event,
     ) -> None:
         if target == "state" and "dirty" in event.keys:
             dirty = event.keys["dirty"]["newValue"]
@@ -361,17 +383,22 @@ class RoomManager:
         # e.g. a new cell added to a notebook
         document.unobserve()
         document.observe(
-            partial(self.on_document_change, file_id, file_type, file_format, document)
+            partial(self.on_document_change, file_id, file_type, file_format, document, room)
         )
         if file_id in self.savers:
             self.savers[file_id].cancel(raise_exception=False)
         self.savers[file_id] = create_task(
-            self.maybe_save_document(file_id, file_type, file_format, document),
+            self.maybe_save_document(file_id, file_type, file_format, document, room),
             self.task_group,
         )
 
     async def maybe_save_document(
-        self, file_id: str, file_type: str, file_format: str, document: YBaseDoc
+        self,
+        file_id: str,
+        file_type: str,
+        file_format: str,
+        document: YBaseDoc,
+        room: YRoom,
     ) -> None:
         # save after configured delay to prevent too frequent saving
         await sleep(self.config.document_save_delay)
@@ -386,6 +413,8 @@ class RoomManager:
         if self.last_modified[file_id] < to_datetime(model.last_modified):
             # file changed on disk, let's revert
             document.source = model.content
+            assert room.ystore is not None
+            await room.ystore.encode_state_as_update(room.ydoc)
             self.last_modified[file_id] = to_datetime(model.last_modified)
             return
         if model.content != document.source:
